# __main__.py
"""
Entry point for running the epub_to_text module as a script.

This allows the module to be run with:
python -m epub_to_text [arguments]
"""

from epub_to_text.cli import main

if __name__ == "__main__":
    main()